@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_ui.c,v 1.81 2004/01/17 09:48:50 fuyu Exp $
+ * $Id: boardlist_ui.c,v 1.83 2004/01/27 15:02:40 fuyu Exp $
  */
 
 #include "config.h"
@@ -227,6 +227,18 @@ open_threadlist_view_cb(GtkWidget *selector, OchushaBulletinBoard *board,
 static guint new_thread_netstat_id;
 
 
+static void
+setup_initial_favorite_thread_list(OchushaBulletinBoard *board,
+				   OchushaApplication *application)
+{
+  if (board->thread_list == NULL)
+    ochusha_bulletin_board_read_threadlist_xml(board,
+					       &application->config,
+					       application->session_subdir,
+					       FALSE);
+}
+
+
 static void
 ensure_my_folder(OchushaApplication *application)
 {
@@ -248,16 +260,10 @@ ensure_my_folder(OchushaApplication *application)
 						 application);
       ochusha_board_category_add_board(application->my_folder,
 				       application->favorites);
+      application->favorites_list
+	= g_slist_prepend(application->favorites_list, application->favorites);
     }
 
-#if 1
-  if (application->favorites->thread_list == NULL)
-    ochusha_bulletin_board_read_threadlist_xml(application->favorites,
-					       &application->config,
-					       application->session_subdir,
-					       FALSE);
-#endif
-
   if (application->all_threads == NULL)
     {
       BulletinBoardGUIInfo *board_info;
@@ -271,6 +277,25 @@ ensure_my_folder(OchushaApplication *application)
 				       application->all_threads);
     }
 
+  if (application->thread_list_already_read)
+    return;
+
+  application->thread_list_already_read = TRUE;
+
+#if 1	/* ɲäȤ硢ΰѰդƤɬפ롣
+	 * ΰ١λɤ߹Ǥɬפ˱ɤ߹褦ˤ
+	 * ȤԲǽǤϤʤޥɥ
+	 */
+  g_slist_foreach(application->favorites_list,
+		  (GFunc)setup_initial_favorite_thread_list, application);
+#else
+  if (application->favorites->thread_list == NULL)
+    ochusha_bulletin_board_read_threadlist_xml(application->favorites,
+					       &application->config,
+					       application->session_subdir,
+					       FALSE);
+#endif
+
 #if 1
   if (application->all_threads->thread_list == NULL)
     ochusha_bulletin_board_read_threadlist_xml(application->all_threads,
@@ -522,6 +547,17 @@ update_board_list(gpointer list_data, gpointer user_data)
 }
 
 
+static void
+save_favorites(VirtualBoard *favorites, OchushaApplication *application)
+{
+  g_return_if_fail(IS_VIRTUAL_BOARD(favorites));
+
+  g_slist_foreach(favorites->board_list, update_board_list, NULL);
+  virtual_board_clean_up_threadlist(favorites, FALSE);
+  write_threadlist(favorites, application);
+}
+
+
 void
 finalize_boardlist(OchushaApplication *application)
 {
@@ -556,6 +592,10 @@ finalize_boardlist(OchushaApplication *application)
     }
 
   /* XXX: Ĥɲä򥵥ݡȤ顢ͤɬפ */
+#if 1
+  g_slist_foreach(application->favorites_list,
+		  (GFunc)save_favorites, application);
+#else
   if (IS_VIRTUAL_BOARD(application->favorites))
     {
       VirtualBoard *favorites = VIRTUAL_BOARD(application->favorites);
@@ -563,6 +603,7 @@ finalize_boardlist(OchushaApplication *application)
       virtual_board_clean_up_threadlist(favorites, FALSE);
       write_threadlist(favorites, application);
     }
+#endif
 
   if (IS_VIRTUAL_BOARD(application->all_threads))
     {
@@ -602,6 +643,14 @@ write_gtkrc_for_tree_view_font(OchushaApplication *application,
 	  fprintf(file, "}\n\n");
 	  fprintf(file, "class \"ThreadlistView\" style \"ochusha-tree-view-style\"\n");
 	  fprintf(file, "class \"BoardlistView\" style \"ochusha-tree-view-style\"\n");
+	  if (application->thread_view_font_name != NULL)
+	    {
+	      fprintf(file, "\n\nstyle \"ochusha-bookmark-style\"\n{\n");
+	      fprintf(file, "  font_name=\"%s\"\n",
+		      application->thread_view_font_name);
+	      fprintf(file, "}\n\n");
+	      fprintf(file, "widget \"*.mybookmark.GtkLabel\" style \"ochusha-bookmark-style\"\n");
+	    }
 	  fclose(file);
 	  return TRUE;
 	}
@@ -1305,10 +1354,16 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook,
     {
       OCHU_OBJECT_UNREF(buffer);
 
+#if 1
+      if (IS_VIRTUAL_BOARD(board))
+	virtual_board_clean_up_threadlist(VIRTUAL_BOARD(board),
+					  board == application->all_threads);
+#else
       if (board == application->favorites)
 	virtual_board_clean_up_threadlist(VIRTUAL_BOARD(board), FALSE);
       else if (board == application->all_threads)
 	virtual_board_clean_up_threadlist(VIRTUAL_BOARD(board), TRUE);
+#endif
     }
 
   if (info->message_id != 0)
@@ -1405,10 +1460,22 @@ board_new_cb(OchushaBBSTable *table, int bbs_type, const gchar *name,
     {
       OchushaBulletinBoard *board
 	= virtual_board_new(name, base_url, application);
+#if 1
+      if (strcmp(base_url, "http:///all-threads") == 0)
+	application->all_threads = board;
+      else
+	{
+	  if (strcmp(base_url, "http:///favorites") == 0)
+	    application->favorites = board;
+	  application->favorites_list
+	    = g_slist_append(application->favorites_list, board);
+	}
+#else
       if (strcmp(base_url, "http:///favorites") == 0)
 	application->favorites = board;
       else if (strcmp(base_url, "http:///all-threads") == 0)
 	application->all_threads = board;
+#endif
       return board;
     }
   return NULL;	/* ¾ΤϤǤ */
