@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha.h,v 1.14 2004/01/18 04:50:26 fuyu Exp $
+ * $Id: ochusha.h,v 1.15 2004/01/27 11:58:11 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_H_
@@ -89,6 +89,11 @@ struct _OchushaConfig
   char *proxy_password;
 
 
+  /* chunksize */
+  int threadlist_chunksize;
+  int thread_chunksize;
+
+
   /* network˥åȤ̤ */
   gboolean offline;
 };
