@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_async_buffer.c,v 1.30 2004/01/27 11:58:11 fuyu Exp $
+ * $Id: ochusha_async_buffer.c,v 1.31 2004/01/27 17:44:53 fuyu Exp $
  */
 
 #include "config.h"
@@ -1333,5 +1333,6 @@ ochusha_async_buffer_emit_access_failed(OchushaAsyncBuffer *buffer,
 {
   g_return_if_fail(OCHUSHA_IS_ASYNC_BUFFER(buffer));
   g_signal_emit(G_OBJECT(buffer),
-		async_buffer_signals[ACCESS_FAILED_SIGNAL], 0);
+		async_buffer_signals[ACCESS_FAILED_SIGNAL], 0,
+		reason_code, reason);
 }
