@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_board_2ch.c,v 1.37 2004/01/21 08:41:01 fuyu Exp $
+ * $Id: ochusha_board_2ch.c,v 1.39 2004/01/27 11:58:11 fuyu Exp $
  */
 
 #include "config.h"
@@ -52,7 +52,7 @@
 #include <zlib.h>
 
 
-#define ANALYSIS_INTERVAL	200
+#define ANALYSIS_INTERVAL	50
 
 
 static void ochusha_board_2ch_class_init(OchushaBoard2chClass *klass);
@@ -866,8 +866,9 @@ ochusha_board_2ch_get_threadlist_source(OchushaBulletinBoard *board,
 
   board_2ch = OCHUSHA_BOARD_2CH(board);
   return ochusha_network_broker_read_from_url(broker, buffer, url,
-					      board_2ch->last_modified, mode,
-					      FALSE);
+					board_2ch->last_modified, mode,
+					FALSE,
+					broker->config->threadlist_chunksize);
 }
 
 
@@ -1009,6 +1010,7 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
   {
     unsigned int offset = 0;
     gboolean buffer_fixed = FALSE;
+    int interval = ANALYSIS_INTERVAL;
 
     if (start_parsing_cb != NULL)
       (*start_parsing_cb)(callback_data);
@@ -1019,7 +1021,6 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 	char *cur_pos = buffer_top + offset;
 	unsigned int rest_of_data = buffer->length - offset;
 	char *eol_pos = NULL;
-	int interval = ANALYSIS_INTERVAL;
 
 	while (rest_of_data > 0
 	       && interval-- > 0
@@ -1110,7 +1111,7 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 		if (each_thread_cb != NULL
 		    && !(*each_thread_cb)(thread, callback_data))
 		  {
-		    break;
+		    goto terminated;
 		  }
 	      }
 
@@ -1135,6 +1136,10 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 	    goto terminated;
 	  }
 
+#if 0
+	fprintf(stderr, "interval=%d\n", interval);
+#endif
+
 	if (interval > 0)
 	  {
 	    size_t old_len = buffer->length;
@@ -1166,7 +1171,7 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 #endif
 	    OCHU_OBJECT_REF(buffer);
 	    g_object_set_qdata(G_OBJECT(buffer), idle_checker_id, buffer);
-	    g_idle_add_full(G_PRIORITY_HIGH_IDLE + 30,
+	    g_idle_add_full(G_PRIORITY_HIGH_IDLE + 15,
 			    advance_parsing, buffer, NULL);
 	    if (before_wait_cb != NULL)
 	      (*before_wait_cb)(callback_data);
@@ -1185,11 +1190,11 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 					  idle_checker_id) != NULL);
 	    if (after_wait_cb != NULL)
 	      (*after_wait_cb)(callback_data);
+	    interval = ANALYSIS_INTERVAL;
 #if 0
 	    fprintf(stderr, "count=%d\n", count);
 #endif
 	  }
-	interval = ANALYSIS_INTERVAL;
 
 	if (status->state == OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_DIRTY)
 	  {
