@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.c,v 1.50 2004/01/22 07:48:32 fuyu Exp $
+ * $Id: ochusha_network_broker.c,v 1.53 2004/01/27 11:58:11 fuyu Exp $
  */
 
 #include "config.h"
@@ -58,6 +58,7 @@
 #define ENABLE_GZIPPED_TRANSFER			1
 #define ENABLE_GZIPPED_DIFFERENCIAL_READ	0
 
+
 #define CACHE_COMPARE_SIZE	128	/* Ĥѹǽˤ٤ */
 #define HEADER_BUFFER_LENGTH	256
 #define LOG_MESSAGE_SIZE	4096
@@ -150,9 +151,33 @@ ochusha_network_broker_class_init(OchushaNetworkBrokerClass *klass)
 }
 
 
+#if ENABLE_NET_CONTEXT
+static void
+ochusha_network_broker_main(WorkerThread *employee, gpointer args)
+{
+  OchushaNetworkBroker *broker = (OchushaNetworkBroker *)args;
+
+  g_main_loop_run(broker->main_loop);
+}
+#endif
+
+
 static void
 ochusha_network_broker_init(OchushaNetworkBroker *broker)
 {
+#if ENABLE_NET_CONTEXT
+  WorkerJob *job = G_NEW0(WorkerJob, 1);
+
+  broker->context = g_main_context_new();
+  broker->main_loop = g_main_loop_new(broker->context, FALSE);
+
+  job->canceled = FALSE;
+  job->job = ochusha_network_broker_main;
+  job->args = broker;
+
+  commit_job(job);
+#endif
+
   broker->config = NULL;
 }
 
@@ -160,6 +185,21 @@ ochusha_network_broker_init(OchushaNetworkBroker *broker)
 static void
 ochusha_network_broker_finalize(GObject *object)
 {
+  OchushaNetworkBroker *broker = (OchushaNetworkBroker *)object;
+
+  if (broker->main_loop != NULL)
+    {
+      g_main_loop_quit(broker->main_loop);
+      g_main_loop_unref(broker->main_loop);
+      broker->main_loop = NULL;
+    }
+
+  if (broker->context != NULL)
+    {
+      g_main_context_unref(broker->context);
+      broker->context = NULL;
+    }
+
   if (G_OBJECT_CLASS(parent_class)->finalize)
     (*G_OBJECT_CLASS(parent_class)->finalize)(object);
 }
@@ -176,6 +216,16 @@ ochusha_network_broker_new(OchushaConfig *config)
 }
 
 
+void
+ochusha_network_broker_terminate(OchushaNetworkBroker *broker)
+{
+  g_return_if_fail(OCHUSHA_IS_NETWORK_BROKER(broker));
+
+  /* event sourceƤinterruptꤹ٤ */
+  g_main_loop_quit(broker->main_loop);
+}
+
+
 void
 ochusha_network_broker_output_log(OchushaNetworkBroker *broker,
 				  const gchar *message)
@@ -379,6 +429,7 @@ get_content_encoding_type(ghttp_request *request)
 typedef struct _NetworkBrokerJobArgs
 {
   int length;
+  int chunksize;
   char *url;
   char *if_modified_since;
 
@@ -404,6 +455,7 @@ struct _WorkerSyncObject
 {
   GObject parent_object;
 
+  OchushaNetworkBroker *broker;
   pthread_mutex_t lock;
   pthread_cond_t cond;
   guint timeout_id;
@@ -516,9 +568,12 @@ worker_sync_object_finalize(GObject *object)
 
 
 static WorkerSyncObject *
-worker_sync_object_new(void)
+worker_sync_object_new(OchushaNetworkBroker *broker)
 {
-  return OCHUSHA_WORKER_SYNC_OBJECT(g_object_new(OCHUSHA_TYPE_WORKER_SYNC_OBJECT, NULL));
+  WorkerSyncObject *sync_object
+    = OCHUSHA_WORKER_SYNC_OBJECT(g_object_new(OCHUSHA_TYPE_WORKER_SYNC_OBJECT, NULL));
+  sync_object->broker = broker;
+  return sync_object;
 }
 
 
@@ -565,7 +620,8 @@ timeout_cb(WorkerSyncObject *sync_object)
   UNLOCK_WORKER_SYNC_OBJECT(sync_object);
 
 #if DEBUG_NETWORK_MOST
-  fprintf(stderr, "leaving: timeout_cb(%p)\n", sync_object);
+  fprintf(stderr, "leaving: timeout_cb(%p) count=%d\n",
+	  sync_object, sync_object->timeout_count);
 #endif
 
   if (!finished)
@@ -613,19 +669,89 @@ poll_cb(GIOChannel *channel, GIOCondition condition,
 }
 
 
+static guint
+ochusha_network_broker_io_add_watch_full(OchushaNetworkBroker *broker,
+					 GIOChannel *channel,
+					 int priority,
+					 GIOCondition condition,
+					 GIOFunc io_func,
+					 gpointer user_data,
+					 GDestroyNotify notify)
+{
+#if ENABLE_NET_CONTEXT
+  GSource *source;
+  guint id;
+
+  g_return_val_if_fail(OCHUSHA_IS_NETWORK_BROKER(broker), 0);
+  g_return_val_if_fail(io_func != NULL, 0);
+
+  source = g_io_create_watch(channel, condition);
+
+  if (priority != G_PRIORITY_DEFAULT)
+    g_source_set_priority(source, priority);
+  g_source_set_callback(source, (GSourceFunc)io_func, user_data, notify);
+
+  id = g_source_attach(source, broker->context);
+  g_source_unref(source);
+
+  return id;
+#else
+  g_return_val_if_fail(OCHUSHA_IS_NETWORK_BROKER(broker), 0);
+  g_return_val_if_fail(io_func != NULL, 0);
+
+  return g_io_add_watch_full(channel, priority, condition,
+			     io_func, user_data, notify);
+#endif
+}
+
+
+static guint
+ochusha_network_broker_timeout_add_full(OchushaNetworkBroker *broker,
+					int priority,
+					guint interval,
+					GSourceFunc timeout_func,
+					gpointer user_data,
+					GDestroyNotify notify)
+{
+#if ENABLE_NET_CONTEXT
+  GSource *source;
+  guint id;
+
+  g_return_val_if_fail(OCHUSHA_IS_NETWORK_BROKER(broker), 0);
+  g_return_val_if_fail(timeout_func != NULL, 0);
+
+  source = g_timeout_source_new(interval);
+  if (priority != G_PRIORITY_DEFAULT)
+    g_source_set_priority(source, priority);
+
+  g_source_set_callback(source, timeout_func, user_data, notify);
+  id = g_source_attach(source, broker->context);
+  g_source_unref(source);
+
+  return id;
+#else
+  g_return_val_if_fail(OCHUSHA_IS_NETWORK_BROKER(broker), 0);
+  g_return_val_if_fail(timeout_func != NULL, 0);
+
+  return g_timeout_add_full(priority, interval,
+			    timeout_func, user_data, notify);
+#endif
+}
+
+
 static void
 register_polling_function_for_read(WorkerSyncObject *sync_object, int fd)
 {
-  LOCK_WORKER_SYNC_OBJECT(sync_object);
   if (sync_object->poll_id == 0)
     {
       GIOChannel *channel = g_io_channel_unix_new(fd);
       OCHU_OBJECT_REF(sync_object);
       sync_object->poll_id
-	= g_io_add_watch_full(channel,
-			      G_PRIORITY_DEFAULT,
-			      G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
-			      (GIOFunc)poll_cb, sync_object, NULL);
+	= ochusha_network_broker_io_add_watch_full(
+				sync_object->broker, channel,
+				G_PRIORITY_DEFAULT,
+				G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
+				(GIOFunc)poll_cb, sync_object, NULL);
       g_io_channel_unref(channel);
 #if DEBUG_NETWORK_MOST
       fprintf(stderr,
@@ -634,7 +760,6 @@ register_polling_function_for_read(WorkerSyncObject *sync_object, int fd)
 #endif
     }
   sync_object->read_ok = FALSE;
-  UNLOCK_WORKER_SYNC_OBJECT(sync_object);
 }
 
 
@@ -672,6 +797,10 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
     force_gunzip = FALSE;
 
   request = ghttp_request_new();
+#if 1
+  if (args->chunksize > 0)
+    ghttp_set_chunksize(request, args->chunksize);
+#endif
   args->request = request;	/* 塹Τˡġ */
 
   ghttp_set_uri(request, (char *)url);
@@ -697,7 +826,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 
   if (!setup_common_request_headers(broker, request, FALSE))
     {
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+      ochusha_async_buffer_emit_access_failed(buffer,
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_INVALID_PROXY,
 			_("Proxy setting may be wrong."));
 
@@ -711,7 +840,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       return FALSE;
     }    
 
-  g_signal_emit_by_name(G_OBJECT(buffer), "access_started");
+  ochusha_async_buffer_emit_access_started(buffer);
 
   snprintf(message, LOG_MESSAGE_SIZE, _("Starting GET request: %s\n"), url);
   ochusha_network_broker_output_log(broker, message);
@@ -741,14 +870,13 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       else if ((read_ok = sync_object->read_ok)
 	       || (state == ghttp_not_done && sock_fd == -1))
 	{
-	  sync_object->read_ok = FALSE;
 	  UNLOCK_WORKER_SYNC_OBJECT(sync_object);
 	  state = ghttp_process(request);
 	  current_status = ghttp_get_status(request);
 	}
       else
 	{
-	  if (sync_object->read_ok)
+	  if (read_ok)
 	    sync_object->timeout_count = 0;
 	  else if (sync_object->timeout_count > POLLING_TIMEOUT_COUNT)
 	    {
@@ -760,7 +888,10 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 
 	      snprintf(message, LOG_MESSAGE_SIZE,
 		       _("Access Timeout: %s\n"), url);
-	      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+#if 0
+	      fprintf(stderr, "Access Timeout(%p): %s\n", sync_object, url);
+#endif
+	      ochusha_async_buffer_emit_access_failed(buffer,
 			    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_TIMEOUT,
 			    message);
 	      ochusha_network_broker_output_log(broker, message);
@@ -861,7 +992,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       if (body_len > 0)
 	{
 	  if (current_status.bytes_total > 0)
-	    g_signal_emit_by_name(G_OBJECT(buffer), "access_progressed",
+	    ochusha_async_buffer_emit_access_progressed(buffer,
 				  current_status.bytes_read,
 				  current_status.bytes_total);
 
@@ -988,6 +1119,10 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       if (state == ghttp_done)
 	break;
 
+      LOCK_WORKER_SYNC_OBJECT(sync_object);
+      sync_object->read_ok = FALSE;
+      sync_object->timeout_count = 0;
+
       if (current_status.proc > ghttp_proc_request)
 	{
 	  sock_fd = ghttp_get_socket(request);
@@ -995,8 +1130,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 	    register_polling_function_for_read(sync_object, sock_fd);
 	}
 
-      LOCK_WORKER_SYNC_OBJECT(sync_object);
-      if (!sync_object->interrupted && !sync_object->read_ok
+      if (!sync_object->interrupted
 	  && pthread_cond_wait(&sync_object->cond, &sync_object->lock) != 0)
 	{
 	  fprintf(stderr, "Couldn't wait a condition.\n");
@@ -1027,7 +1161,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       else
 	status->date = NULL;
 
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_finished");
+      ochusha_async_buffer_emit_access_finished(buffer);
 
       ghttp_request_destroy(request);
       args->request = NULL;
@@ -1054,7 +1188,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       snprintf(message, LOG_MESSAGE_SIZE,
 	       _("Access failed(%s): status_code=%d (%s)\n"),
 	       url, status_code, error_message);
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+      ochusha_async_buffer_emit_access_failed(buffer,
 			    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 			    message);
 
@@ -1085,7 +1219,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
     {
       snprintf(message, LOG_MESSAGE_SIZE, _("Access failed(%s): %d (%s)\n"),
 	       url, status_code, ghttp_reason_phrase(request));
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+      ochusha_async_buffer_emit_access_failed(buffer,
 			    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 			    message);
       ochusha_network_broker_output_log(broker, message);
@@ -1190,11 +1324,11 @@ ochusha_network_broker_employ_worker_thread(OchushaNetworkBroker *broker,
 					    const char *url,
 					    const char *if_modified_since,
 					    JobFunc *job_function,
-					    gboolean modestly)
+					    gboolean modestly, int chunksize)
 {
   NetworkBrokerJobArgs *args;
   WorkerJob *job;
-  WorkerSyncObject *sync_object = worker_sync_object_new();
+  WorkerSyncObject *sync_object = worker_sync_object_new(broker);
   OchushaNetworkBrokerBufferStatus *status
     = G_NEW0(OchushaNetworkBrokerBufferStatus, 1);
   if (buffer == NULL || !ochusha_async_buffer_reset(buffer))
@@ -1210,10 +1344,12 @@ ochusha_network_broker_employ_worker_thread(OchushaNetworkBroker *broker,
   g_signal_connect_object(G_OBJECT(buffer), "wakeup_now",
 			  G_CALLBACK(wakeup_now_cb), sync_object, 0);
 
-  sync_object->timeout_id = g_timeout_add_full(G_PRIORITY_DEFAULT - 1,
-					       POLLING_INTERVAL_MILLIS,
-					       (GSourceFunc)timeout_cb,
-					       sync_object, NULL);
+  sync_object->timeout_id
+    = ochusha_network_broker_timeout_add_full(sync_object->broker,
+					      G_PRIORITY_DEFAULT - 1,
+					      POLLING_INTERVAL_MILLIS,
+					      (GSourceFunc)timeout_cb,
+					      sync_object, NULL);
 
   status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_UNKNOWN;
 
@@ -1221,6 +1357,7 @@ ochusha_network_broker_employ_worker_thread(OchushaNetworkBroker *broker,
   args->length = buffer->length;
   buffer->length = 0;
   args->url = G_STRDUP(url);
+  args->chunksize = chunksize;
   if (if_modified_since != NULL
       && (args->length > 0
 	  || ochusha_config_cache_file_exist(broker->config, url)))
@@ -1274,7 +1411,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	url = NULL;
 
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+      ochusha_async_buffer_emit_access_failed(buffer,
 		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 		_("Access terminated."));
 
@@ -1358,6 +1495,10 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_HIT;
       request = ghttp_request_new();
+#if 1
+      if (args->chunksize > 0)
+	ghttp_set_chunksize(request, args->chunksize);
+#endif
       args->request = request;
       if (request == NULL)
 	{
@@ -1393,7 +1534,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	{
 	  /* ۤäƥåϤɤΤ */
 	  status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_OK;
-	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+	  ochusha_async_buffer_emit_access_failed(buffer,
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_INVALID_PROXY,
 			_("Proxy setting may be wrong."));
 
@@ -1403,7 +1544,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  goto finish_try_update_cache;
 	}
 
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_started");
+      ochusha_async_buffer_emit_access_started(buffer);
 
       snprintf(message, LOG_MESSAGE_SIZE, _("Updating cache file: %s\n"),
 	       url);
@@ -1423,7 +1564,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
 	  snprintf(message, LOG_MESSAGE_SIZE,
 		   _("Access terminated: %s\n"), url);
-	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+	  ochusha_async_buffer_emit_access_failed(buffer,
 		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 		message);
 	  ochusha_network_broker_output_log(broker, message);
@@ -1447,6 +1588,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  if (sync_object->interrupted)
 	    {
 	      sync_object->interrupted = FALSE;
+	      sync_object->read_ok = FALSE;
 	      UNLOCK_WORKER_SYNC_OBJECT(sync_object);
 	      read_ok = FALSE;
 	    }
@@ -1459,7 +1601,10 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	      current_status = ghttp_get_status(request);
 	    }
 	  else
-	    UNLOCK_WORKER_SYNC_OBJECT(sync_object);
+	    {
+	      sync_object->read_ok = FALSE;
+	      UNLOCK_WORKER_SYNC_OBJECT(sync_object);
+	    }
 
 	  if (status_code == 0
 	      && (current_status.proc == ghttp_proc_response
@@ -1482,7 +1627,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 		  status->state
 		    = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_OK;
 
-		  g_signal_emit_by_name(G_OBJECT(buffer), "access_finished");
+		  ochusha_async_buffer_emit_access_finished(buffer);
 
 		  ghttp_request_destroy(request);
 		  args->request = NULL;
@@ -1568,7 +1713,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  if (body_len > 0)
 	    {
 	      if (current_status.bytes_total > 0)
-		g_signal_emit_by_name(G_OBJECT(buffer), "access_progressed",
+		ochusha_async_buffer_emit_access_progressed(buffer,
 				      current_status.bytes_read,
 				      current_status.bytes_total);
 
@@ -1604,7 +1749,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
 		  snprintf(message, LOG_MESSAGE_SIZE,
 			   _("Access terminated: %s\n"), url);
-		  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+		  ochusha_async_buffer_emit_access_failed(buffer,
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 			message);
 		  ochusha_network_broker_output_log(broker, message);
@@ -1643,7 +1788,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 		  snprintf(message, LOG_MESSAGE_SIZE,
 			   _("Access terminated: %s\n"),
 			   url);
-		  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+		  ochusha_async_buffer_emit_access_failed(buffer,
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 			message);
 		  ochusha_network_broker_output_log(broker, message);
@@ -1655,6 +1800,10 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  if (state == ghttp_done)
 	    break;
 
+	  LOCK_WORKER_SYNC_OBJECT(sync_object);
+	  sync_object->read_ok = FALSE;
+	  sync_object->timeout_count = 0;
+
 	  if (current_status.proc > ghttp_proc_request)
 	    {
 	      sock_fd = ghttp_get_socket(request);
@@ -1662,8 +1811,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 		register_polling_function_for_read(sync_object, sock_fd);
 	    }
 
-	  LOCK_WORKER_SYNC_OBJECT(sync_object);
-	  if (!sync_object->interrupted && !sync_object->read_ok
+	  if (!sync_object->interrupted
 	      && pthread_cond_wait(&sync_object->cond,
 				   &sync_object->lock) != 0)
 	    {
@@ -1689,7 +1837,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	status->date = NULL;
 
       status->status_code = status_code;
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_finished");
+      ochusha_async_buffer_emit_access_finished(buffer);
 
       ghttp_request_destroy(request);
       args->request = NULL;
@@ -1718,7 +1866,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
 	  snprintf(message, LOG_MESSAGE_SIZE,
 		   _("Access terminated: %s\n"), url);
-	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+	  ochusha_async_buffer_emit_access_failed(buffer,
 		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 		message);
 	  ochusha_network_broker_output_log(broker, message);
@@ -1752,7 +1900,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
     {
       snprintf(message, LOG_MESSAGE_SIZE,
 	       _("Access failed(%s): %s\n"), url, error_message);
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+      ochusha_async_buffer_emit_access_failed(buffer,
 			    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 			    message);
       ochusha_network_broker_output_log(broker, message);
@@ -1765,7 +1913,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  snprintf(message, LOG_MESSAGE_SIZE,
 		   _("Access failed(%s): %d (%s)\n"),
 		   url, status_code, ghttp_reason_phrase(request));
-	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+	  ochusha_async_buffer_emit_access_failed(buffer,
 				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 				message);
 	  ochusha_network_broker_output_log(broker, message);
@@ -1775,7 +1923,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  snprintf(message, LOG_MESSAGE_SIZE,
 		   _("Access failed(%s): unknown reason.\n"),
 		   url);
-	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+	  ochusha_async_buffer_emit_access_failed(buffer,
 				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 				message);
 	  ochusha_network_broker_output_log(broker, message);
@@ -1832,7 +1980,7 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   if (!ochusha_async_buffer_active_ref(buffer))
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+      ochusha_async_buffer_emit_access_failed(buffer,
 		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 		_("Access terminated."));
       goto finish_refresh_cache_after_read;
@@ -1842,7 +1990,7 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   if (args == NULL)
     {
       status->state = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_ACCESS_FAILED;
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+      ochusha_async_buffer_emit_access_failed(buffer,
 		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 		_("Access terminated."));
       goto finish_refresh_cache_after_read;
@@ -1860,7 +2008,7 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
   else
     {
       /* ̿Ԥå夬(if_modified_sinceNULL)ʾ */
-      ochusha_async_buffer_reset(buffer);
+      /* ochusha_async_buffer_reset(buffer); */
       ochusha_async_buffer_update_length(buffer, 0);
       if (read_cache_to_buffer(broker->config, args->url, buffer))
 	{
@@ -1869,14 +2017,14 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	    {	/* å夬ä */
 	      if (args->if_modified_since != NULL)
 		status->last_modified = G_STRDUP(args->if_modified_since);
-	      g_signal_emit_by_name(G_OBJECT(buffer), "access_finished");
+	      ochusha_async_buffer_emit_access_finished(buffer);
 	    }
 	  else
 	    {	/* ̿˼ԤΤǻʤåȤ */
 	      snprintf(message, LOG_MESSAGE_SIZE,
 		       _("Use cached file(%s): due to networking error.\n"),
 		       args->url);
-	      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+	      ochusha_async_buffer_emit_access_failed(buffer,
 				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 				message);
 	      ochusha_network_broker_output_log(broker, message);
@@ -1890,7 +2038,7 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	      snprintf(message, LOG_MESSAGE_SIZE,
 		       _("Couldn't find cache file: %s\n"),
 		       args->url);
-	      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+	      ochusha_async_buffer_emit_access_failed(buffer,
 				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 				message);
 	      ochusha_network_broker_output_log(broker, message);
@@ -1898,11 +2046,11 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  else
 	    {
 	      if (buffer->state != OCHUSHA_ASYNC_BUFFER_TERMINATED)
-		g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+		ochusha_async_buffer_emit_access_failed(buffer,
 				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 				_("Couldn't read data via the network."));
 	      else
-		g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+		ochusha_async_buffer_emit_access_failed(buffer,
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 			_("Access terminated."));
 
@@ -1998,7 +2146,7 @@ background_read_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
   if (ochusha_async_buffer_read_file(buffer, fd))
     {
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_finished");
+      ochusha_async_buffer_emit_access_finished(buffer);
     }
   else
     {
@@ -2047,7 +2195,7 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
 				     const char *url,
 				     const char *if_modified_since,
 				     OchushaNetworkBrokerCacheMode mode,
-				     gboolean modestly)
+				     gboolean modestly, int chunksize)
 {
   g_return_val_if_fail(OCHUSHA_IS_NETWORK_BROKER(broker)
 		       && broker->config != NULL && url != NULL, NULL);
@@ -2058,7 +2206,7 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
   if (mode == OCHUSHA_NETWORK_BROKER_CACHE_IGNORE && broker->config->offline)
     {
 #if 0	/* ƤӽФ¦Թǰ褺ȥ */
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+      ochusha_async_buffer_emit_access_failed(buffer,
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_NOT_CONNECTED,
 			_("Network not connected on offline mode."));
 #endif
@@ -2110,7 +2258,7 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
 	{
 	  /* åʤĥå夷Ȥʤ */
 #if 0	/* ƤӽФ¦Թǰ褺ȥ */
-	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+	  ochusha_async_buffer_emit_access_failed(buffer,
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_CACHE_NOT_FOUND,
 			_("Couldn't find cache that should exist."));
 #endif
@@ -2120,7 +2268,7 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
       if (broker->config->offline)
 	{
 #if 0	/* ƤӽФ¦Թǰ褺ȥ */
-	  g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+	  ochusha_async_buffer_emit_access_failed(buffer,
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_NOT_CONNECTED,
 			_("Network not connected on offline mode."));
 #endif
@@ -2134,20 +2282,20 @@ ochusha_network_broker_read_from_url(OchushaNetworkBroker *broker,
       return ochusha_network_broker_employ_worker_thread(broker, buffer,
 						url, if_modified_since,
 						(JobFunc *)try_update_cache,
-						 modestly);
+						modestly, chunksize);
 
     case OCHUSHA_NETWORK_BROKER_CACHE_TRY_REFRESH:
     case OCHUSHA_NETWORK_BROKER_CACHE_AS_IS:
       return ochusha_network_broker_employ_worker_thread(broker, buffer,
 					url, if_modified_since,
 					(JobFunc *)refresh_cache_after_read,
-					modestly);
+					modestly, chunksize);
 
     case OCHUSHA_NETWORK_BROKER_CACHE_IGNORE:
       return ochusha_network_broker_employ_worker_thread(broker, buffer,
 							 url, NULL,
 							 (JobFunc *)force_read,
-							 modestly);
+							 modestly, chunksize);
 
     default:
       break;
@@ -2177,6 +2325,10 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
   ghttp_status state;
 
   request = ghttp_request_new();
+#if 1
+  if (args->chunksize > 0)
+    ghttp_set_chunksize(request, args->chunksize);
+#endif
   args->request = request;	/* 塹Τˡġ */
 
   ghttp_set_uri(request, (char *)url);
@@ -2188,7 +2340,7 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
 
   if (!setup_common_request_headers(broker, request, FALSE))
     {
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+      ochusha_async_buffer_emit_access_failed(buffer,
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_INVALID_PROXY,
 			_("Proxy setting may be wrong."));
 
@@ -2197,7 +2349,7 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
       return FALSE;
     }    
 
-  g_signal_emit_by_name(G_OBJECT(buffer), "access_started");
+  ochusha_async_buffer_emit_access_started(buffer);
 
   ghttp_set_sync(request, ghttp_sync);
   ghttp_prepare(request);
@@ -2241,7 +2393,7 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
     {
       ghttp_current_status current_status = ghttp_get_status(request);
       if (current_status.bytes_total > 0)
-	g_signal_emit_by_name(G_OBJECT(buffer), "access_progressed",
+	ochusha_async_buffer_emit_access_progressed(buffer,
 			      current_status.bytes_read,
 			      current_status.bytes_total);
 
@@ -2320,7 +2472,7 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
 #if DEBUG_NETWORK_MOST
   fprintf(stderr, "http_read_from_url_synchronously succeeded.\n");
 #endif
-  g_signal_emit_by_name(G_OBJECT(buffer), "access_finished");
+  ochusha_async_buffer_emit_access_finished(buffer);
 
   return TRUE;
 
@@ -2330,7 +2482,7 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
 #endif
   if (error_message != NULL)
     {
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+      ochusha_async_buffer_emit_access_failed(buffer,
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 			error_message);
 #if 0
@@ -2343,7 +2495,7 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
       char message[LOG_MESSAGE_SIZE];
       snprintf(message, LOG_MESSAGE_SIZE, "Access failed: %d (%s)\n",
 	       status_code, ghttp_reason_phrase(request));
-      g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+      ochusha_async_buffer_emit_access_failed(buffer,
 			OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 			message);
 #if 0
@@ -2351,7 +2503,7 @@ http_read_from_url_synchronously(OchushaNetworkBroker *broker,
 #endif
     }
   else
-    g_signal_emit_by_name(G_OBJECT(buffer), "access_failed",
+    ochusha_async_buffer_emit_access_failed(buffer,
 			  OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 			  _("Access failed: unknown reason."));
 
@@ -2449,7 +2601,7 @@ ochusha_network_broker_read_from_cgi(OchushaNetworkBroker *broker,
 
   return ochusha_network_broker_employ_worker_thread(broker, buffer, url, NULL,
 					(JobFunc *)force_read_synchronously,
-					FALSE);
+					FALSE, 0);
 }
 
 
