@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.h,v 1.12 2004/01/22 07:48:32 fuyu Exp $
+ * $Id: ochusha_network_broker.h,v 1.14 2004/01/27 11:58:11 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_NETWORK_BROKER_H_
@@ -41,6 +41,7 @@
 extern "C" {
 #endif
 
+#define ENABLE_NET_CONTEXT			1
 
 #define OCHUSHA_TYPE_NETWORK_BROKER		(ochusha_network_broker_get_type())
 #define OCHUSHA_NETWORK_BROKER(obj)		(G_TYPE_CHECK_INSTANCE_CAST((obj), OCHUSHA_TYPE_NETWORK_BROKER, OchushaNetworkBroker))
@@ -81,6 +82,15 @@ struct _OchushaNetworkBroker
   GObject parent_object;
 
   const OchushaConfig *config;
+
+#if ENABLE_NET_CONTEXT
+/*
+ * ͥåȥ˴ؤpollingtimeoutѤGLibŪMain Event Loop
+ * ѥåɤǲ󤷤Ƥߤ¸
+ */
+  GMainContext *context;
+  GMainLoop *main_loop;
+#endif
 };
 
 
@@ -117,6 +127,8 @@ typedef struct _OchushaNetworkBrokerBufferStatus
 GType ochusha_network_broker_get_type(void);
 OchushaNetworkBroker *ochusha_network_broker_new(OchushaConfig *config);
 
+void ochusha_network_broker_terminate(OchushaNetworkBroker *broker);
+
 void ochusha_network_broker_output_log(OchushaNetworkBroker *broker,
 				       const gchar *message);
 
@@ -126,7 +138,7 @@ OchushaAsyncBuffer *ochusha_network_broker_read_from_url(
 					const char *url,
 					const char *if_modified_since,
 					OchushaNetworkBrokerCacheMode mode,
-					gboolean modestly);
+					gboolean modestly, int chunksize);
 OchushaAsyncBuffer *ochusha_network_broker_read_from_cgi(
 					OchushaNetworkBroker *broker,
 					OchushaAsyncBuffer *buffer,
