@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_jbbs.c,v 1.33 2004/01/22 07:48:32 fuyu Exp $
+ * $Id: ochusha_thread_jbbs.c,v 1.34 2004/01/23 18:52:02 fuyu Exp $
  */
 
 #include "config.h"
@@ -1170,17 +1170,21 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 
   if (!ochusha_async_buffer_active_ref(html_buffer))
     {
-      g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_failed",
+#if 0
+      ochusha_async_buffer_emit_access_failed(dat_buffer,
 		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 		_("Access terminated."));
+#endif
       goto terminated;
     }
 
   if (!ochusha_async_buffer_active_ref(dat_buffer))
     {
-      g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_failed",
+#if 0
+      ochusha_async_buffer_emit_access_failed(dat_buffer,
 		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 		_("Access terminated."));
+#endif
       ochusha_async_buffer_active_unref(html_buffer);
       goto terminated;
     }
@@ -1488,7 +1492,9 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 #if DEBUG_HTML_TO_DAT_CONVERTER_MOST
 	  fprintf(stderr, "conversion done.\n");
 #endif
-	  g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_finished");
+#if 0
+	  ochusha_async_buffer_emit_access_finished(dat_buffer);
+#endif
 
 	  if (previous_line != NULL)
 	    {
@@ -1503,9 +1509,11 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 #if DEBUG_HTML_TO_DAT_CONVERTER_MOST
 	  fprintf(stderr, "wait terminated.\n");
 #endif
-	  g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_failed",
+#if 0
+	  ochusha_async_buffer_emit_access_failed(dat_buffer,
 		OCHUSHA_NETWORK_BROKER_FAILURE_REASON_ACCESS_TERMINATED,
 		_("Access terminated."));
+#endif
 	  if (previous_line != NULL)
 	    {
 	      g_free(previous_line);
@@ -1714,7 +1722,7 @@ ochusha_thread_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
     {
       if (dat_buffer != NULL)
 	{
-	  g_signal_emit_by_name(G_OBJECT(dat_buffer), "access_failed",
+	  ochusha_async_buffer_emit_access_failed(dat_buffer,
 			  OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 			  _("Access failed: unknown reason."));
 	  ochusha_async_buffer_fix(dat_buffer);
@@ -1733,6 +1741,19 @@ ochusha_thread_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
 	}
     }
 
+  g_signal_connect_swapped(G_OBJECT(html_buffer), "access_started",
+			G_CALLBACK(ochusha_async_buffer_emit_access_started),
+			dat_buffer);
+  g_signal_connect_swapped(G_OBJECT(html_buffer), "access_progressed",
+		G_CALLBACK(ochusha_async_buffer_emit_access_progressed),
+		dat_buffer);
+  g_signal_connect_swapped(G_OBJECT(html_buffer), "access_finished",
+			G_CALLBACK(ochusha_async_buffer_emit_access_finished),
+			dat_buffer);
+  g_signal_connect_swapped(G_OBJECT(html_buffer), "access_failed",
+			G_CALLBACK(ochusha_async_buffer_emit_access_failed),
+			dat_buffer);
+
   args = G_NEW0(HTML2DATConverterJobArgs, 1);
   args->html_buffer = html_buffer;
   args->dat_buffer = dat_buffer;
